@@ -6,16 +6,16 @@ __version__ = "0.1"
 __license__ = "MIT"
 
 import warnings
-from math import floor
 from os import path, mkdir
 from webbrowser import open as show_file
+
 from scipy.optimize import newton
 
 import matplotlib.pyplot as plt
 from matplotlib.backends.backend_pdf import PdfPages
 import numpy as np
 import pandas as pd
-from tqdm import tqdm
+from tqdm.auto import tqdm
 
 from .arg_parser import heavy_rain_parser
 from .idf_backend import IdfParameters
@@ -25,6 +25,7 @@ from .in_out import import_series
 from .sww_utils import (remove_timezone, guess_freq, rain_events, agg_events, event_duration, resample_rain_series,
                         rain_bar_plot, IdfError, )
 from .plot_helpers import idf_bar_axes
+from .synthetic_rainseries import _BlockRain, _EulerRain
 
 
 ########################################################################################################################
@@ -36,8 +37,15 @@ class IntensityDurationFrequencyAnalyse:
     and calculates the distribution of the rainfall as a function of the return period and the duration
     
     for duration steps up to 12 hours (and more) and return period in a range of '0.5a <= T_n <= 100a'
-    """
 
+    Attributes:
+        _series (pandas.Series): rain time-series
+        _freq (pandas.DateOffset): frequency of the rain series
+        _return_periods_frame (pandas.DataFrame): with return periods of all given durations
+        _rain_events (pandas.DataFrame):
+        _rainfall_sum_frame (pandas.DataFrame): with rain sums of all given durations
+
+    """
     def __init__(self, series_kind=SERIES.PARTIAL, worksheet=METHOD.KOSTRA, extended_durations=False):
         """
         heavy rain as a function of the duration and the return period acc. to DWA-A 531 (2012)
@@ -52,8 +60,8 @@ class IntensityDurationFrequencyAnalyse:
             worksheet (str): ['DWA-A_531', 'ATV-A_121', 'DWA-A_531_advektiv']
             extended_durations (bool): add [720, 1080, 1440, 2880, 4320, 5760, 7200, 8640] minutes to the calculation
         """
-        self._series = None  # type: pd.Series # rain time-series
-        self._freq = None  # frequency of the rain series
+        self._series = None
+        self._freq = None
 
         #  how to calculate the idf curves
         # IdfParameters.__init__(self, series_kind=series_kind, worksheet=worksheet,
@@ -62,19 +70,19 @@ class IntensityDurationFrequencyAnalyse:
                                         extended_durations=extended_durations)
         # self._parameter = None  # type: IdfParameters #  how to calculate the idf curves
 
-        self._return_periods_frame = None  # type: pd.DataFrame # with return periods of all given durations
+        self._return_periods_frame = None
         self._rain_events = None
-        self._rainfall_sum_frame = None  # type: pd.DataFrame # with rain sums of all given durations
+        self._rainfall_sum_frame = None
 
     # __________________________________________________________________________________________________________________
     @property
-    def series(self):
+    def series(self) -> pd.Series:
         if self._series is None:
             raise IdfError('No Series defined for IDF-Analysis!')
         return self._series
 
     @series.setter
-    def series(self, series):
+    def series(self, series: pd.Series):
         self._series = series
 
     def set_series(self, series):
@@ -106,6 +114,7 @@ class IntensityDurationFrequencyAnalyse:
     def duration_steps(self):
         """
         get duration steps (in minutes) for the parameter calculation and basic evaluations
+
         Returns:
             list | numpy.ndarray: duration steps in minutes
         """
@@ -155,19 +164,24 @@ class IntensityDurationFrequencyAnalyse:
 
     def read_parameters(self, filename):
         """
-        read parameters from a .yaml-file to save computation time.
-        extract interim results from parameters
+        Read parameters from a .yaml-file to save computation time.
+
+        Extract interim results from parameters.
 
         Args:
-            filename (str): filename of the parameters yaml-file
+            filename (str, Path): filename of the parameters yaml-file
         """
         self._parameter = IdfParameters.from_yaml(filename)
 
     def auto_save_parameters(self, filename):
-        """auto-save the parameters as a yaml-file to save computation time."""
+        """Auto-save the parameters as a yaml-file to save computation time."""
         if path.isfile(filename):
             self.read_parameters(filename)
         else:
+            from os import makedirs
+            parent = path.dirname(filename)
+            if parent and not path.isdir(parent):
+                makedirs(parent)
             self.write_parameters(filename)
 
     # __________________________________________________________________________________________________________________
@@ -256,7 +270,7 @@ class IntensityDurationFrequencyAnalyse:
     # __________________________________________________________________________________________________________________
     def result_table(self, durations=None, return_periods=None, add_names=False):
         """
-        get a standard idf table of rainfall depth with return periods as columns and durations as rows
+        Get an idf-table of rainfall depth with return periods as columns and durations as rows.
 
         Args:
             durations (list | numpy.ndarray): list of durations in minutes for the table
@@ -272,7 +286,7 @@ class IntensityDurationFrequencyAnalyse:
         if return_periods is None:
             return_periods = [1, 2, 3, 5, 10, 20, 25, 30, 50, 75, 100]
 
-        result_table = dict()
+        result_table = {}
         for t in return_periods:
             result_table[t] = self.depth_of_rainfall(durations, t)
 
@@ -287,6 +301,23 @@ class IntensityDurationFrequencyAnalyse:
     ####################################################################################################################
     def result_figure(self, min_duration=5.0, max_duration=720.0, logx=False, return_periods=None, color=True, ax=None,
                       linestyle=None, add_interim=False):
+        """
+        Create a plot with the idf-curves with duration on the x-axis and rainfall depth on the y-axis.
+
+        Args:
+            min_duration (float): Shortest duration on the plot.
+            max_duration (float): Longest duration on the plot.
+            logx (bool): Use a logarithmic scale on the x-axis.
+            return_periods (list[int] | Optional): List of return periods to plot. Default = [1, 2, 5, 10, 50, 100]
+            color (bool): Use color and a legend to differentiate between the return periods (default).
+                Otherwise, annotation text and black lines.
+            ax (plt.Axes): Axes to plot on. Default = create new one.
+            linestyle (str): Line-style for the plotted lines.
+            add_interim (bool): Add interim results from the series analysis as scatter points.
+
+        Returns:
+            (plt.Figure, plt.Axes): figure and axes of the plot.
+        """
         duration_steps = np.arange(min_duration, max_duration + 1, 1)
 
         if return_periods is None:
@@ -412,14 +443,16 @@ class IntensityDurationFrequencyAnalyse:
     ####################################################################################################################
     def get_rainfall_sum_frame(self, series=None, durations=None):
         """
+        Get a rainfall sum frame for any series with the duration steps as columns.
+
+        Default: The time-series and the duration-steps of the analysis.
 
         Args:
-            series (pandas.Series):
-            durations (list): list of durations in minutes which are of interest (default: pre defined
-            durations)
+            series (pandas.Series, Optional): rainfall time-series
+            durations (list, Optional): list of durations in minutes which are of interest (default: pre-defined durations)
 
         Returns:
-            pandas.DataFrame: rain sum depending of the duration per datetimeindex
+            pandas.DataFrame: Rain sum depending on the duration per datetime-index.
         """
         if durations is None:
             durations = self.duration_steps
@@ -436,7 +469,7 @@ class IntensityDurationFrequencyAnalyse:
             # ts = series.replace(0, np.NaN).dropna()
 
         df = pd.DataFrame(index=ts.index)
-        # df = dict()
+        # df = {}
 
         freq_num = delta2min(freq)
 
@@ -455,10 +488,10 @@ class IntensityDurationFrequencyAnalyse:
     @property
     def rainfall_sum_frame(self):
         """
-        get the return periods over the whole time-series for the default duration steps.
+        Get the rainfall sum over the whole time-series for the default duration steps.
 
         Returns:
-            pandas.DataFrame: data-frame of return periods where the columns are the duration steps
+            pandas.DataFrame: Rain sum depending on the duration per datetime-index.
         """
         if self._rainfall_sum_frame is None:
             self._rainfall_sum_frame = self.get_rainfall_sum_frame()
@@ -467,25 +500,28 @@ class IntensityDurationFrequencyAnalyse:
     ####################################################################################################################
     def get_return_periods_frame(self, series=None, durations=None):
         """
+        Get the return periods for any time-series with the duration steps as columns.
+
+        Default: The time-series and the duration-steps of the analysis.
 
         Args:
-            series (pandas.Series):
-            durations (list): list of durations in minutes which are of interest (default: pre defined durations)
+            series (pandas.Series, Optional): rainfall time-series
+            durations (list, Optional): Durations in minutes which are of interest (default: pre-defined durations)
 
         Returns:
-            pandas.DataFrame: return periods depending of the duration per datetimeindex
+            pandas.DataFrame: Return periods depending on the duration per datetime-index.
         """
         sums = self.get_rainfall_sum_frame(series=series, durations=durations)
         df = pd.DataFrame(index=sums.index)
-        # df = dict()
+        # df = {}
         for d in frame_looper(sums.index.size, columns=sums.columns, label='return_periods'):
-            df[d] = self.get_return_period(height_of_rainfall=sums[d][sums[d] > 0.1], duration=d)
+            df[d] = self.get_return_period(height_of_rainfall=sums[d][sums[d] >= 0.1], duration=d)
         return df#.fillna(0)#.round(2)
 
     @property
     def return_periods_frame(self):
         """
-        get the return periods over the whole time-series for the default duration steps.
+        Get the return periods over the whole time-series for the default duration steps.
 
         Returns:
             pandas.DataFrame: data-frame of return periods where the columns are the duration steps
@@ -556,11 +592,13 @@ class IntensityDurationFrequencyAnalyse:
 
     def add_max_return_periods_to_events(self, events):
         if COL.MAX_PERIOD not in events:
-            max_periods = self.return_periods_frame.max(axis=1)
-            max_periods_duration = self.return_periods_frame.idxmax(axis=1)
+            return_periods_frame = self.return_periods_frame
+            max_periods = return_periods_frame.max(axis=1).fillna(0)  # fill NaN -> weil < 0.1 gefiltert wurde
+            max_periods_duration = return_periods_frame.idxmax(axis=1)
             datetime_max = agg_events(events, max_periods, 'idxmax')
-            # TODO: where are values NaN ???
             datetime_max = np.where(np.isnan(datetime_max), events[COL.START].values, datetime_max)
+            # alternative:
+            # [xv if c else yv for c, xv, yv in zip(np.isnan(datetime_max), events[COL.START].values, datetime_max)]
             events[COL.MAX_PERIOD] = max_periods[datetime_max].values
             events[COL.MAX_PERIOD_DURATION] = max_periods_duration[datetime_max].values
 
@@ -576,7 +614,6 @@ class IntensityDurationFrequencyAnalyse:
             min_event_rain_sum (float): only events with a bigger rain sum will be created
             min_return_period (float): only events with a bigger return period will be analysed
                                        (the plot will be created anyway)
-            out_path (str): path and filename of the final report
             durations (list[int]): analysed durations
                         (default: [5, 10, 15, 20, 30, 45, 60, 90, 120, 180, 240, 360, 540, 720, 1080, 1440, 2880, 4320])
         """
@@ -609,17 +646,14 @@ class IntensityDurationFrequencyAnalyse:
 
         pdf.close()
 
-    def event_plot(self, event, durations=None, unit='mm', column_name='Precipitation', min_return_period=0.5):
+    def event_plot(self, event, durations=None, unit='mm', column_name='Precipitation', min_return_period=1.):
         if isinstance(event, pd.Series):
             event = event.to_dict()
-        start = event[COL.START]
-        end = event[COL.END]
 
-        plot_range = slice(start - pd.Timedelta(self._freq), end + pd.Timedelta(self._freq))
-
-        return_periods_frame = self.return_periods_frame[plot_range]
+        plot_range = slice(event[COL.START] - pd.Timedelta(self._freq), event[COL.END] + pd.Timedelta(self._freq))
 
         if COL.MAX_PERIOD not in event:
+            return_periods_frame = self.return_periods_frame[plot_range]
             event[COL.MAX_PERIOD] = return_periods_frame.max().max()
             event[COL.MAX_PERIOD_DURATION] = return_periods_frame.max().idxmax()
 
@@ -638,10 +672,10 @@ class IntensityDurationFrequencyAnalyse:
                 max_dur = max(self.duration_steps)
 
             return_periods_frame_extended = self.get_return_periods_frame(
-                self.series[start - pd.Timedelta(minutes=max_dur):
-                            end + pd.Timedelta(self._freq)].resample(self._freq).sum(),
+                self.series[event[COL.START] - pd.Timedelta(minutes=max_dur):
+                            event[COL.END] + pd.Timedelta(self._freq)].resample(self._freq).sum(),
                 durations=durations
-            )
+            ).round(1)
 
             idf_bar_ax = fig.add_subplot(211)
             idf_bar_ax = idf_bar_axes(idf_bar_ax, return_periods_frame_extended)
@@ -679,7 +713,7 @@ class IntensityDurationFrequencyAnalyse:
     def return_period_event_figure(self, event):
         period_line = self.return_periods_frame[event[COL.START]:event[COL.END]].max()
 
-        period_line[period_line < 0.75] = np.NaN
+        # period_line[period_line < 0.75] = np.NaN
         period_line = period_line.dropna()
 
         ax = period_line.plot()  # type: plt.Axes
@@ -695,9 +729,49 @@ class IntensityDurationFrequencyAnalyse:
                                                    event[COL.END] - event[COL.START],
                                                    period_line.max(),
                                                    period_line.idxmax()))
-
+        # ax.set_xscale('log')
+        # ax.set_yscale('log')
+        # print(ax.get_ylim())
+        # ax.set_ylim(0.01, 300)
+        # exit()
         ax.set_xticks(period_line.index)
         ax.set_xticklabels([minutes_readable(m) for m in period_line.index])
         ax.set_xlabel('duration steps')
         ax.set_ylabel('return period in years')
         return ax.get_figure(), ax
+
+    @classmethod
+    def from_idf_table(cls, idf_table, worksheet=METHOD.KOSTRA):
+        """
+        Create an IDF-analysis-object based on an idf-tabel (i.e. from a given KOSTRA table)
+
+        Args:
+            idf_table (pandas.DataFrame): idf-table with index=Durations and columns=return Period and values=Rainheight
+            worksheet (str | optional): name of the worksheet to use.  default: 'KOSTRA'
+
+        Returns:
+            IntensityDurationFrequencyAnalyse: idf-object
+        """
+        idf = cls(worksheet=worksheet)
+        idf._parameter.reverse_engineering(idf_table)
+        return idf
+
+    @property
+    def model_rain_block(self):
+        """
+        Create a model block rain class.
+
+        Returns:
+            _BlockRain: Synthetic model block rain.
+        """
+        return _BlockRain(self)
+
+    @property
+    def model_rain_euler(self):
+        """
+        Create a model Euler rain class.
+
+        Returns:
+            _EulerRain: Synthetic model Euler rain.
+        """
+        return _EulerRain(self)
